class Talk2DomError(Exception): ...


class AuthError(Talk2DomError): ...


class RateLimitError(Talk2DomError): ...


class RemoteError(Talk2DomError): ...


class BadRequestError(Talk2DomError): ...
